@@ -1,24 +1,26 @@
-import pygtk
-pygtk.require('2.0')
-import gtk
+#!/usr/bin/env python
 import time
-from matplotlib.matlab import *
+from pylab import *
 
 def get_memory():
     "Simulate a function that returns system memory"
-    return 100*(1+sin(0.5*pi*time.time()))
+    return 100*(0.5+0.5*sin(0.5*pi*time.time()))
 
 def get_cpu():
     "Simulate a function that returns cpu usage"
-    return 100*(1+sin(0.2*pi*(time.time()-0.25)))
+    return 100*(0.5+0.5*sin(0.2*pi*(time.time()-0.25)))
 
 def get_net():
     "Simulate a function that returns network bandwidth"
-    return 100*(1+sin(0.7*pi*(time.time()-0.1)))
+    return 100*(0.5+0.5*sin(0.7*pi*(time.time()-0.1)))
 
 def get_stats():
     return get_memory(), get_cpu(), get_net()
 
+# turn interactive mode on for dynamic updates.  If you aren't in
+# interactive mode, you'll need to use a GUI event handler/timer.
+ion()  
+
 fig = figure(1)
 ax = subplot(111)
 ind = arange(1,4)
@@ -34,20 +36,17 @@ ax.set_xticklabels(['Memory', 'CPU', 'Bandwidth'])
 ax.set_ylabel('Percent usage')
 ax.set_title('System Monitor')
 
-def updatefig(*args):
+for i in range(200):  # run for a little while
     m,c,n = get_stats()
+
     pm.set_height(m)
     pc.set_height(c)
     pn.set_height(n)
     ax.set_ylim([0,100])
 
-    fig.draw()
-    
-    return gtk.TRUE
+    draw()
 
 
-gtk.timeout_add(250, updatefig)
-show()
     
 
 
