@@ -143,13 +143,20 @@ the exception of those in mlab.py provided by matplotlib.
 from __future__ import generators
 
 
-__version__  = '0.86'
-__revision__ = '$Revision: 1.92 $'
-__date__     = '$Date: 2006/01/06 13:59:47 $'
+__version__  = '0.86.2'
+__revision__ = '$Revision: 1.104 $'
+__date__     = '$Date: 2006/01/24 12:56:45 $'
 
-import sys, os, warnings, shutil
+import sys, os, warnings, shutil, md5
 import distutils.sysconfig
 
+# Needed for toolkit setuptools support
+try:
+    __import__('pkg_resources').declare_namespace(__name__)
+except ImportError:
+    pass # must not have setuptools
+
+
 if not hasattr(sys, 'argv'):  # for modpython
     sys.argv = ['modpython']
 
@@ -181,10 +188,10 @@ try:
 except ImportError: _havedate = False
 else: _havedate = True
 
-try:
-    import pkg_resources # pkg_resources is part of setuptools
-except ImportError: _have_pkg_resources = False
-else: _have_pkg_resources = True
+#try:
+#    import pkg_resources # pkg_resources is part of setuptools
+#except ImportError: _have_pkg_resources = False
+#else: _have_pkg_resources = True
 
 if not _python23:
     def enumerate(seq):
@@ -429,6 +436,74 @@ def _get_data_path():
 get_data_path = verbose.wrap('matplotlib data path %s', _get_data_path, always=False)
 
 
+def checkdep_dvipng():
+    try:
+        stdin, stdout = os.popen4('dvipng -version')
+        line = stdout.readlines()[1]
+        v = line.split()[-1]
+        float(v)
+        return v
+    except IndexError, ValueError:
+        return None
+
+def checkdep_ghostscript():
+    try:
+        if sys.platform == 'win32':
+            command = 'gswin32c -v'
+        else:
+            command = 'gs -v'
+        stdin, stdout = os.popen4(command)
+        line = stdout.readlines()[0]
+        v = line.split()[2]
+        vtest = '.'.join(v.split('.')[:2]) # deal with version numbers like '7.07.1'
+        float(vtest)
+        return vtest
+    except IndexError, ValueError:
+        return None
+
+def checkdep_ps2eps():
+    try:
+        stdin, stdout = os.popen4('ps2eps -v')
+        line = stdout.readlines()[-1]
+        v = line.split()[-1]
+        float(v)
+        return v
+    except IndexError, ValueError:
+        return None
+
+def checkdep_tex():
+    try:
+        stdin, stdout = os.popen4('tex -v')
+        line = stdout.readlines()[0]
+        v = 0.0 # no version installed until we find one...
+        for potential_version_numstr in line.split():
+            if potential_version_numstr.startswith('3.1'):
+                v = potential_version_numstr
+                float(v)
+                break # found version number
+        return v
+    except IndexError, ValueError:
+        return None
+
+def checkdep_xpdf():
+    try:
+        stdin, stdout = os.popen4('xpdf -v')
+        line = stdout.readlines()[0]
+        v = line.split()[-1]
+        float(v)
+        return v
+    except IndexError, ValueError:
+        return None
+
+def compare_versions(a, b):
+    "return True if a is greater than b"
+    if a:
+        a = [int(i) for i in a.split('.')]
+        b = [int(i) for i in b.split('.')]
+        if a[0]>b[0]: return True
+        elif (a[0]==b[0]) and (a[1]>=b[1]): return True
+        else: return False
+    else: return False
 
 def validate_path_exists(s):
     'If s is a path, return s, else False'
@@ -567,8 +642,6 @@ def validate_verbose_fileo(s):
         try: fileo = file(s, 'w')
         except IOError:
             raise ValueError('Verbose object could not open log file "%s" for writing.\nCheck your matplotlibrc verbose.fileo setting'%s)
-
-
         else:
             verbose.fileo = fileo
     return verbose.fileo
@@ -581,8 +654,92 @@ validate_ps_papersize = ValidateInStrings([
         'c0', 'c1', 'c2', 'c3', 'c4', 'c5', 'c6'
         ], ignorecase=True)
 
+def validate_ps_distiller(s):
+    s = s.lower()
+    
+    if s == 'none':
+        return None
+    elif s == 'false':
+        return False
+    
+    flag = True
+    
+    if (s == 'ghostscript') or (s == 'xpdf'):
+        gs_req = '7.07'
+        gs_sugg = '8.16'
+        gs_v = checkdep_ghostscript()
+        if compare_versions(gs_v, gs_sugg): pass
+        elif compare_versions(gs_v, gs_req):
+            warnings.warn( 'ghostscript-%s.%s found. ghostscript-%s.%s or later is \
+recommended to use the ps.usedistiller option.' % (gs_v, gs_sugg), 'helpful')
+        else:
+            flag = False
+            warnings.warn('matplotlibrc ps.usedistiller option can not be used \
+unless ghostscript-%s or later is installed on your system'% gs_req)
+    
+    if s == 'xpdf':
+        xpdf_req = '3.0'
+        ps2eps_req = '1.58'
+        xpdf_v = checkdep_xpdf()
+        if compare_versions(xpdf_v, xpdf_req): pass
+        else:
+            flag = False
+            warnings.warn('matplotlibrc ps.usedistiller can not be set to xpdf \
+unless xpdf-%s or later is installed on your system' % xpdf_req)
+        
+        ps2eps_v = checkdep_ps2eps()
+        if compare_versions(ps2eps_v, ps2eps_req): pass
+        else:
+            flag = False
+            warnings.warn('matplotlibrc ps.usedistiller can not be set to xpdf \
+unless ps2eps-%s or later is installed on your system' % ps2eps_req)
+        
+        if flag: return s.lower()
+        else: return None
+    else: 
+        raise ValueError('matplotlibrc ps.usedistiller must either be none, \
+ghostscript or xpdf')
+
 validate_tex_engine = ValidateInStrings(['tex', 'latex'], ignorecase=True)
 
+def validate_usetex(s):
+    bl = validate_bool(s)
+    if bl:
+        tex_req = '3.1415'
+        gs_req = '7.07'
+        gs_sugg = '8.16'
+        dvipng_req = '1.5'
+        flag = True
+        
+        tex_v = checkdep_tex()
+        if compare_versions(tex_v, tex_req): pass
+        else:
+            flag = False
+            warnings.warn('matplotlibrc text.usetex option can not be used \
+unless TeX-%s or later is installed on your system' % tex_req)
+        
+        dvipng_v = checkdep_dvipng()
+        if compare_versions(dvipng_v, dvipng_req): pass
+        else:
+            flag = False
+            warnings.warn( 'matplotlibrc text.usetex can not be used with *Agg \
+backend unless dvipng-1.5 or later is installed on your system')
+        
+        gs_v = checkdep_ghostscript()
+        if compare_versions(gs_v, gs_sugg): pass
+        elif compare_versions(gs_v, gs_req):
+            warnings.warn( 'ghostscript-%s found. ghostscript-%s or later is \
+recommended for use with the text.usetex option.' % (gs_v, gs_sugg))
+        else:
+            flag = False
+            warnings.warn('matplotlibrc text.usetex can not be used \
+unless ghostscript-%s or later is installed on your system'% gs_req)
+        
+        if flag: return True
+        else: return False
+    else:
+        return bl
+
 validate_joinstyle = ValidateInStrings(['miter', 'round', 'bevel'], ignorecase=True)
 
 validate_capstyle = ValidateInStrings(['butt', 'round', 'projecting'], ignorecase=True)
@@ -615,7 +772,6 @@ class ValidateInterval:
 
 
 
-
 # a map from key -> value, converter
 defaultParams = {
     'backend'           : ['GTK', str],
@@ -667,7 +823,7 @@ defaultParams = {
 
     # text props
     'text.color'        : ['k', validate_color],     # black
-    'text.usetex'       : [False, validate_bool],
+    'text.usetex'       : [False, validate_usetex],
     'text.tex.engine'   : ['tex', validate_tex_engine], # TeX or LaTeX
     'text.fontstyle'    : ['normal', str],
     'text.fontangle'    : ['normal', str],
@@ -694,6 +850,20 @@ defaultParams = {
 
     'polaraxes.grid'         : [True, validate_bool],   # display polar grid or not
 
+    #legend properties
+    'legend.isaxes'	:	[True,validate_bool],
+    'legend.numpoints' 	:	[ 4,validate_int],      # the number of points in the legend line
+    'legend.fontsize' : ["small",validate_fontsize],
+    'legend.pad' 	:	[ 0.2, validate_float],         # the fractional whitespace inside the legend border
+    'legend.markerscale' 	:	[ 0.6, validate_float],    # the relative size of legend markers vs. original
+    # the following dimensions are in axes coords
+    'legend.labelsep' 	:	[ 0.005, validate_float],    # the vertical space between the legend entries
+    'legend.handlelen' 	:	[ 0.05, validate_float],  # the length of the legend lines
+    'legend.handletextsep' 	:	[ 0.02, validate_float], # the space between the legend line and legend text
+    'legend.axespad' 	:	[ 0.02, validate_float], # the border between the axes and legend edge
+    'legend.shadow' : [ False, validate_bool ],
+
+
     # tick properties
     'xtick.major.size'   : [5, validate_float],      # major xtick size in points
     'xtick.minor.size'   : [2, validate_float],      # minor xtick size in points
@@ -740,7 +910,7 @@ defaultParams = {
     'tk.pythoninspect'   : [ False, validate_bool],  # Set PYTHONINSPECT
     'ps.papersize'      : [ 'letter', validate_ps_papersize], # Set the papersize/type
     'ps.useafm'   : [ False, validate_bool],  # Set PYTHONINSPECT
-    'ps.usedistiller'   : [ False, validate_bool],  # use ghostscript to distill ps output
+    'ps.usedistiller'   : [ False, validate_ps_distiller],  # use ghostscript or xpdf to distill ps output
     'ps.distiller.res'  : [6000, validate_int],       # dpi
     'plugins.directory' : ['.matplotlib_plugins', str], # where plugin directory is locate
 
@@ -830,7 +1000,7 @@ WARNING: Old rc filename "%s" found and renamed to
 
 
 
-def rc_params():
+def rc_params(fail_on_error=False):
     'Return the default params updated from the values in the rc file'
 
     deprecated_map = {
@@ -878,13 +1048,18 @@ def rc_params():
         ind = val.find('#')
         if ind>=0: val = val[:ind]   # ignore trailing comments
         val = val.strip()
-        try: cval = converter(val)   # try to convert to proper type or raise
-        except Exception, msg:
-            warnings.warn('Bad val "%s" on line #%d\n\t"%s"\n\tin file "%s"\n\t%s' % (val, cnt, line, fname, msg))
-            continue
-        else:
-            # Alles Klar, update dict
+        if fail_on_error:
+            cval = converter(val)   # try to convert to proper type or raise
             defaultParams[key][0] = cval
+        else:
+            try: cval = converter(val)   # try to convert to proper type or raise
+            except Exception, msg:
+                warnings.warn('Bad val "%s" on line #%d\n\t"%s"\n\tin file "%s"\n\t%s' % (
+                    val, cnt, line, fname, msg))
+                continue
+            else:
+                # Alles Klar, update dict
+                defaultParams[key][0] = cval
 
     # strip the conveter funcs and return
     ret =  dict([ (key, tup[0]) for key, tup in defaultParams.items()])
@@ -1057,3 +1232,30 @@ verbose.report('verbose.level %s'%verbose.level)
 verbose.report('interactive is %s'%rcParams['interactive'])
 verbose.report('platform is %s'%sys.platform)
 verbose.report('loaded modules: %s'%sys.modules.keys(), 'debug')
+
+class ExampleInfo:
+    pass
+
+class ExampleManager:
+    baseurl = 'http://matplotlib.sf.net'
+    urls = ['%s/%s'%(baseurl, subdir) for subdir in
+            ( 'examples', 'examples/widgets')]
+
+    def get_examples(self):
+        import urllib, re
+        rgx = re.compile('.*<A HREF="([^.]+\.py)">.*')
+        examples = []
+        for url in urls:
+            lines = urllib.urlopen(url).readlines()
+            for line in lines:
+                m = rgx.match(line)
+                if m is not None:
+                    examples.append('%s/%s'%(url, m.group(1)))
+        return examples
+            
+    def get_info(self, s):
+        """
+        return an ExampleInfo instance from s, the string content of
+        an example
+        """
+        pass
