@@ -34,6 +34,7 @@ class Artist:
         self._transform = identity_transform()
         self._transformSet = False
         self._visible = True
+        self._animated = False
         self._alpha = 1.0
         self.clipbox = None
         self._clipon = False
@@ -52,7 +53,7 @@ class Artist:
         self._oid += 1
         return oid
 
-    def remove_callback(self, oid):        
+    def remove_callback(self, oid):
         try: del self._propobservers[oid]
         except KeyError: pass
 
@@ -60,21 +61,21 @@ class Artist:
         'fire event when property changed'
         for oid, func in self._propobservers.items():
             func(self)
-            
+
     def is_transform_set(self):
-        'Artist has transform explicity let'                
+        'Artist has transform explicity let'
         return self._transformSet
 
     def set_transform(self, t):
         """
-set the Transformation instance used by this artist
+        set the Transformation instance used by this artist
 
-ACCEPTS: a matplotlib.transform transformation instance
-"""
+        ACCEPTS: a matplotlib.transform transformation instance
+        """
         self._transform = t
         self._transformSet = True
         self.pchanged()
-        
+
     def get_transform(self):
         'return the Transformation instance used by this artist'
         return self._transform
@@ -85,36 +86,40 @@ ACCEPTS: a matplotlib.transform transformation instance
     def get_figure(self):
         'return the figure instance'
         return self.figure
-    
+
     def set_figure(self, fig):
         """
-Set the figure instance the artist belong to
+        Set the figure instance the artist belong to
 
-ACCEPTS: a matplotlib.figure.Figure instance
+        ACCEPTS: a matplotlib.figure.Figure instance
         """
         self.figure = fig
         self.pchanged()
-        
-    def set_clip_box(self, clipbox):        
-        """
-Set the artist's clip Bbox
 
-ACCEPTS: a matplotlib.transform.Bbox instance
+    def set_clip_box(self, clipbox):
+        """
+        Set the artist's clip Bbox
+
+        ACCEPTS: a matplotlib.transform.Bbox instance
         """
         self.clipbox = clipbox
         self._clipon = clipbox is not None
         self.pchanged()
-        
+
     def get_alpha(self):
         """
-Return the alpha value used for blending - not supported on all
-backends
+        Return the alpha value used for blending - not supported on all
+        backends
         """
         return self._alpha
-    
+
     def get_visible(self):
         "return the artist's visiblity"
-        return self._visible 
+        return self._visible
+
+    def get_animated(self):
+        "return the artist's animated state"
+        return self._animated
 
     def get_clip_on(self):
         'Return whether artist uses clipping'
@@ -126,47 +131,58 @@ backends
 
     def set_clip_on(self, b):
         """
-Set  whether artist uses clipping
+        Set  whether artist uses clipping
 
-ACCEPTS: [True | False]
-"""
+        ACCEPTS: [True | False]
+        """
         self._clipon = b
         if not b: self.clipbox = None
         self.pchanged()
+
     def draw(self, renderer, *args, **kwargs):
         'Derived classes drawing method'
-        if not self.get_visible(): return 
+        if not self.get_visible(): return
 
     def set_alpha(self, alpha):
         """
-Set the alpha value used for blending - not supported on
-all backends
+        Set the alpha value used for blending - not supported on
+        all backends
 
-ACCEPTS: float
+        ACCEPTS: float
         """
         self._alpha = alpha
         self.pchanged()
 
     def set_lod(self, on):
         """
-Set Level of Detail on or off.  If on, the artists may examine
-things like the pixel width of the axes and draw a subset of
-their contents accordingly
+        Set Level of Detail on or off.  If on, the artists may examine
+        things like the pixel width of the axes and draw a subset of
+        their contents accordingly
 
-ACCEPTS: [True | False]
+        ACCEPTS: [True | False]
         """
         self._lod = on
         self.pchanged()
-        
+
     def set_visible(self, b):
         """
-set the artist's visiblity
+        set the artist's visiblity
 
-ACCEPTS: [True | False]
-"""
+        ACCEPTS: [True | False]
+        """
         self._visible = b
         self.pchanged()
 
+
+    def set_animated(self, b):
+        """
+        set the artist's animation state
+
+        ACCEPTS: [True | False]
+        """
+        self._animated = b
+        self.pchanged()
+
     def update(self, props):
         store = self.eventson
         self.eventson = False
@@ -179,17 +195,17 @@ ACCEPTS: [True | False]
             changed = True
         self.eventson = store
         if changed: self.pchanged()
-        
+
 
     def get_label(self):
         return self._label
 
     def set_label(self, s):
         """
-Set the line label to s for auto legend
+        Set the line label to s for auto legend
 
-ACCEPTS: any string
-"""
+        ACCEPTS: any string
+        """
         self._label = s
         self.pchanged()
 
@@ -197,13 +213,13 @@ ACCEPTS: any string
 
     def set_zorder(self, level):
         """
-Set the zorder for the artist
+        Set the zorder for the artist
 
-ACCEPTS: any number
-"""
+        ACCEPTS: any number
+        """
         self.zorder = level
         self.pchanged()
-    
+
     def update_from(self, other):
         'copy properties from other to self'
         self._transform = other._transform
@@ -241,7 +257,7 @@ class ArtistInspector:
         sequence (all Artists are of the same type) and it is your
         responsibility to make sure this is so.
         """
-        if iterable(o): o = o[0]
+        if iterable(o) and len(o): o = o[0]
         self.o = o
         self.aliasd = self.get_aliases()
 
@@ -381,7 +397,7 @@ def getp(o, *args):
 
       o.get_somename()
 
-    get can be used to query all the gettable properties with get(o)
+    getp can be used to query all the gettable properties with getp(o)
     Many properties have aliases for shorter typing, eg 'lw' is an
     alias for 'linewidth'.  In the output, aliases and full property
     names will be listed as
@@ -404,38 +420,33 @@ def getp(o, *args):
     return func()
 
 def get(o, *args, **kwargs):
-    return geto(o, *args, **kwargs)
+    return getp(o, *args, **kwargs)
 get.__doc__ = getp.__doc__
-                        
-def set(*args, **kwargs):
-    message = 'set deprecated because it overrides python2.4 builtin set.  Use setp'
-    warnings.warn(message, DeprecationWarning, stacklevel=2)
 
-    return setp(*args, **kwargs)
-    
+
 def setp(h, *args, **kwargs):
     """
-    matlab(TM) and pylab allow you to use set and get to set and get
-    object properties, as well as to do introspection on the object
-    For example, to set the linestyle of a line to be dashed, you can do
+    matplotlib supports the use of setp ("set property") and getp to set and get object properties, as well as to do
+    introspection on the object For example, to set the linestyle of a
+    line to be dashed, you can do
 
       >>> line, = plot([1,2,3])
-      >>> set(line, linestyle='--')
+      >>> setp(line, linestyle='--')
 
     If you want to know the valid types of arguments, you can provide the
     name of the property you want to set without a value
 
-      >>> set(line, 'linestyle')
+      >>> setp(line, 'linestyle')
           linestyle: [ '-' | '--' | '-.' | ':' | 'steps' | 'None' ]
 
     If you want to see all the properties that can be set, and their
     possible values, you can do
 
 
-      >>> set(line)
+      >>> setp(line)
           ... long output listing omitted'
 
-    set operates on a single instance or a list of instances.  If you are
+    setp operates on a single instance or a list of instances.  If you are
     in quey mode introspecting the possible values, only the first
     instance in the sequnce is used.  When actually setting values, all
     the instances will be set.  Eg, suppose you have a list of two lines,
@@ -445,13 +456,13 @@ def setp(h, *args, **kwargs):
         >>> y1 = sin(2*pi*x)
         >>> y2 = sin(4*pi*x)
         >>> lines = plot(x, y1, x, y2)
-        >>> set(lines, linewidth=2, color='r')
+        >>> setp(lines, linewidth=2, color='r')
 
-    Set works with the matlab(TM) style string/value pairs or with python
-    kwargs.  For example, the following are equivalent
+    Set works with the matlab(TM) style string/value pairs or with
+    python kwargs.  For example, the following are equivalent
 
-        >>> set(lines, 'linewidth', 2, 'color', r')  # matlab style
-        >>> set(lines, linewidth=2, color='r')       # python style
+        >>> setp(lines, 'linewidth', 2, 'color', r')  # matlab style
+        >>> setp(lines, linewidth=2, color='r')       # python style
     """
 
     insp = ArtistInspector(h)
@@ -486,5 +497,5 @@ def setp(h, *args, **kwargs):
     return [x for x in flatten(ret)]
 
 
-    
-    
+
+
