@@ -314,24 +314,24 @@ def get_split_ind(seq, N):
      
   
 def wrap(prefix, text, cols):
+    'wrap text with prefix at length cols'
+    pad = ' '*len(prefix.expandtabs())
+    available = cols - len(pad)
 
-   pad = ' '*len(prefix.expandtabs())
-   available = cols - len(pad)
+    seq = text.split(' ')
+    Nseq = len(seq)
+    ind = 0
+    lines = []
+    while ind<Nseq:
+        lastInd = ind
+        ind += get_split_ind(seq[ind:], available)
+        lines.append(seq[lastInd:ind])
 
-   seq = text.split(' ')
-   Nseq = len(seq)
-   ind = 0
-   lines = []
-   while ind<Nseq:
-      lastInd = ind
-      ind += get_split_ind(seq[ind:], available)
-      lines.append(seq[lastInd:ind])
-
-   # add the prefix to the first line, pad with spaces otherwise
-   ret = prefix + ' '.join(lines[0]) + '\n'
-   for line in lines[1:]:
-      ret += pad + ' '.join(line) + '\n'
-   return ret
+    # add the prefix to the first line, pad with spaces otherwise
+    ret = prefix + ' '.join(lines[0]) + '\n'
+    for line in lines[1:]:
+        ret += pad + ' '.join(line) + '\n'
+    return ret
 
 
 
@@ -438,12 +438,6 @@ def allpairs(x):
     return [ (s, f) for i, f in enumerate(x) for s in x[i+1:] ]
 
         
-if __name__=='__main__':
-    assert( allequal([1,1,1]) )
-    assert(not  allequal([1,1,0]) )
-    assert( allequal([]) )
-    assert( allequal(('a', 'a')))
-    assert( not allequal(('a', 'b')))    
 
 
 # python 2.2 dicts don't have pop
@@ -471,6 +465,23 @@ def popd(d, *args):
     return val
 
 
+class maxdict(dict):
+    """
+    A dictionary with a maximum size; this doesn't override all the
+    relevant methods to contrain size, just setitem, so use with
+    caution
+    """
+    def __init__(self, maxsize):
+        dict.__init__(self)
+        self.maxsize = maxsize
+        self._killkeys = []
+    def __setitem__(self, k, v):
+        if len(self)>=self.maxsize:
+            del self[self._killkeys[0]]
+            del self._killkeys[0]
+        dict.__setitem__(self, k, v)
+        self._killkeys.append(k)        
+
 
 
 class Stack:
@@ -566,3 +577,11 @@ def finddir(o, match, case=False):
 def reverse_dict(d):
     'reverse the dictionary -- may lose data if values are not uniq!'
     return dict([(v,k) for k,v in d.items()])
+
+
+if __name__=='__main__':
+    assert( allequal([1,1,1]) )
+    assert(not  allequal([1,1,0]) )
+    assert( allequal([]) )
+    assert( allequal(('a', 'a')))
+    assert( not allequal(('a', 'b')))    
