@@ -394,6 +394,7 @@ class ContourSet(ScalarMappable, ContourLabeler):
         self.extent = kwargs.get('extent', None)
         cmap = kwargs.get('cmap', None)
         self.colors = kwargs.get('colors', None)
+        norm = kwargs.get('norm', None)
         self.clip_ends = kwargs.get('clip_ends', True)
         self.antialiased = kwargs.get('antialiased', True)
         self.nchunk = kwargs.get('nchunk', 0)
@@ -417,18 +418,19 @@ class ContourSet(ScalarMappable, ContourLabeler):
         self.cl = []
         self.cl_cvalues = []
 
-        ScalarMappable.__init__(self, cmap = cmap) # sets self.cmap;
-                                                   # default norm for now
+        kw = {'cmap': cmap}
+        if norm is not None:
+            kw['norm'] = norm
+        ScalarMappable.__init__(self, **kw) # sets self.cmap;
         self._process_colors()
 
-
         if self.filled:
             if self.linewidths is None:
                 self.linewidths = 0.05 # Good default for Postscript.
             if iterable(self.linewidths):
                 self.linewidths = self.linewidths[0]
             #C = _contour.Cntr(x, y, z.filled(), z.mask())
-            C = _contour.Cntr(x, y, z.filled(), ma.getmask(z))
+            C = _contour.Cntr(x, y, z.filled(), ma.getmaskorNone(z))
             lowers = self.levels[:-1]
             uppers = self.levels[1:]
             for level, level_upper, color in zip(lowers, uppers, self.tcolors):
@@ -444,7 +446,7 @@ class ContourSet(ScalarMappable, ContourLabeler):
         else:
             tlinewidths = self._process_linewidths()
             #C = _contour.Cntr(x, y, z.filled(), z.mask())
-            C = _contour.Cntr(x, y, z.filled(), ma.getmask(z))
+            C = _contour.Cntr(x, y, z.filled(), ma.getmaskorNone(z))
             for level, color, width in zip(self.levels, self.tcolors, tlinewidths):
                 nlist = C.trace(level, points = 1)
                 col = LineCollection(nlist)
@@ -453,7 +455,6 @@ class ContourSet(ScalarMappable, ContourLabeler):
 
                 if level < 0.0 and self.monochrome:
                     col.set_linestyle((0, (6.,6.)),)
-                    #print "setting dashed"
                 col.set_label(str(level))         # only for self-documentation
                 self.ax.add_collection(col)
                 self.collections.append(col)
@@ -687,6 +688,11 @@ class ContourSet(ScalarMappable, ContourLabeler):
             * cmap = None: a cm Colormap instance from matplotlib.cm.
               - if cmap == None and colors == None, a default Colormap is used.
 
+            * norm = None: a matplotlib.colors.normalize instance for
+              scaling data values to colors.
+              - if norm == None, and colors == None, the default
+                linear scaling is used.
+
             * origin = None: 'upper'|'lower'|'image'|None.
               If 'image', the rc value for image.origin will be used.
               If None (default), the first value of Z will correspond
