@@ -10,7 +10,8 @@ from artist import Artist
 from colors import normalize, colorConverter
 import cm
 import numerix
-from numerix import arange, asarray, UInt8
+import numerix.ma as ma
+from numerix import arange, asarray, UInt8, Float32, repeat, NewAxis, typecode
 import _image
 
 
@@ -37,9 +38,9 @@ class AxesImage(Artist, cm.ScalarMappable):
         extent is a data xmin, xmax, ymin, ymax for making image plots
         registered with data plots.  Default is the image dimensions
         in pixels
-        
+
         """
-        Artist.__init__(self)        
+        Artist.__init__(self)
         cm.ScalarMappable.__init__(self, norm, cmap)
 
         if origin is None: origin = rcParams['image.origin']
@@ -83,15 +84,15 @@ class AxesImage(Artist, cm.ScalarMappable):
         self._aspectdr = dict([ (v,k) for k,v in self._aspectd.items()])
 
         if aspect is None: aspect = rcParams['image.aspect']
-        if interpolation is None: interpolation = rcParams['image.interpolation']        
-        
+        if interpolation is None: interpolation = rcParams['image.interpolation']
+
         self.set_interpolation(interpolation)
         self.set_aspect( aspect)
         self.axes = ax
 
 
         self._imcache = None
-        
+
     def get_size(self):
         'Get the numrows, numcols of the input image'
         if self._A is None:
@@ -101,10 +102,10 @@ class AxesImage(Artist, cm.ScalarMappable):
 
     def set_alpha(self, alpha):
         """
-Set the alpha value used for blending - not supported on
-all backends
+        Set the alpha value used for blending - not supported on
+        all backends
 
-ACCEPTS: float
+        ACCEPTS: float
         """
         Artist.set_alpha(self, alpha)
         self._imcache = None
@@ -117,10 +118,10 @@ ACCEPTS: float
         self._imcache = None
         cm.ScalarMappable.changed(self)
 
-    def make_image(self, flipy):
+    def make_image(self):
         if self._A is not None:
             if self._imcache is None:
-                if self._A.typecode() == UInt8:
+                if typecode(self._A) == UInt8:
                     im = _image.frombyte(self._A, 0)
                 else:
                     x = self.to_rgba(self._A, self._alpha)
@@ -133,11 +134,15 @@ ACCEPTS: float
 
 
         bg = colorConverter.to_rgba(self.axes.get_frame().get_facecolor(), 0)
+
+        if self.origin=='upper':
+            im.flipud_in()
+
         im.set_bg( *bg)
         im.is_grayscale = (self.cmap.name == "gray" and
                            len(self._A.shape) == 2)
-        
-        im.set_aspect(self._aspectd[self._aspect])        
+
+        im.set_aspect(self._aspectd[self._aspect])
         im.set_interpolation(self._interpd[self._interpolation])
 
 
@@ -146,23 +151,26 @@ ACCEPTS: float
         numrows, numcols = im.get_size()
         im.reset_matrix()
 
-        xmin, xmax, ymin, ymax = self.get_extent() 
+        xmin, xmax, ymin, ymax = self.get_extent()
         dxintv = xmax-xmin
         dyintv = ymax-ymin
 
         # the viewport scale factor
         sx = dxintv/self.axes.viewLim.width()
         sy = dyintv/self.axes.viewLim.height()
-            
+
         if im.get_interpolation()!=_image.NEAREST:
             im.apply_translation(-1, -1)
 
         # the viewport translation
         tx = (xmin-self.axes.viewLim.xmin())/dxintv * numcols
-        if flipy: 
-            ty = -(ymax-self.axes.viewLim.ymax())/dyintv * numrows
-        else:
-            ty = (ymin-self.axes.viewLim.ymin())/dyintv * numrows
+
+
+        #if flipy:
+        #    ty = -(ymax-self.axes.viewLim.ymax())/dyintv * numrows
+        #else:
+        #    ty = (ymin-self.axes.viewLim.ymin())/dyintv * numrows
+        ty = (ymin-self.axes.viewLim.ymin())/dyintv * numrows
 
         l, b, widthDisplay, heightDisplay = self.axes.bbox.get_bounds()
 
@@ -174,53 +182,47 @@ ACCEPTS: float
         rx = widthDisplay / numcols
         ry = heightDisplay  / numrows
 
-        
+
         if im.get_aspect()==_image.ASPECT_PRESERVE:
             if ry < rx: rx = ry
             # todo: center the image in viewport
             im.apply_scaling(rx, rx)
-            
+
         else:
             im.apply_scaling(rx, ry)
 
         #print tx, ty, sx, sy, rx, ry, widthDisplay, heightDisplay
         im.resize(int(widthDisplay+0.5), int(heightDisplay+0.5),
                   norm=self._filternorm, radius=self._filterrad)
+
+        if self.origin=='upper':
+            im.flipud_in()
+
         return im
-    
+
+
     def draw(self, renderer, *args, **kwargs):
-
-        if not self.get_visible(): return 
-        isUpper = self.origin=='upper'
-        flipy = renderer.flipy()
-
-        im = self.make_image(isUpper)
+        if not self.get_visible(): return
+        im = self.make_image()
         l, b, widthDisplay, heightDisplay = self.axes.bbox.get_bounds()
-        if isUpper:
-            #offset is distance from top of figure
-            oy = self.axes.figure.bbox.height()-(b+heightDisplay)
-            renderer.draw_image(l, oy, im, self.origin, self.axes.bbox)
-        else:
-            # compute the location of the origin
-            oy = b
-            renderer.draw_image(l, oy, im, self.origin, self.axes.bbox)
+        renderer.draw_image(l, b, im, self.axes.bbox)
 
     def write_png(self, fname):
         """Write the image to png file with fname"""
         im = self.make_image(False)
         im.write_png(fname)
-        
+
 
     def set_data(self, A, shape=None):
         """
-Set the image array
+        Set the image array
 
-ACCEPTS: numeric/numarray/PIL Image A"""
+        ACCEPTS: numeric/numarray/PIL Image A"""
         # check if data is PIL Image without importing Image
         if hasattr(A,'getpixel'): X = pil_to_array(A)
-        else: X = asarray(A) # assume array
-        
-        if (X.typecode() != UInt8
+        else: X = ma.asarray(A) # assume array
+
+        if (typecode(X) != UInt8
             or len(X.shape) != 3
             or X.shape[2] > 4
             or X.shape[2] < 3):
@@ -232,9 +234,9 @@ ACCEPTS: numeric/numarray/PIL Image A"""
 
     def set_array(self, A):
         """
-retained for backwards compatibility - use set_data instead
+        retained for backwards compatibility - use set_data instead
 
-ACCEPTS: numeric/numarray/PIL Image A"""
+        ACCEPTS: numeric/numarray/PIL Image A"""
 
 
         self.set_data(A)
@@ -243,7 +245,7 @@ ACCEPTS: numeric/numarray/PIL Image A"""
         """
         Return the method used to constrain the aspoect ratio of the
         One of
-        
+
         'free'     : aspect ratio not constrained
         'preserve' : preserve aspect ratio when resizing
         """
@@ -256,7 +258,7 @@ ACCEPTS: numeric/numarray/PIL Image A"""
         Return the interpolation method the image uses when resizing.
 
         One of
-        
+
         'bicubic', 'bilinear', 'blackman100', 'blackman256', 'blackman64',
         'nearest', 'sinc144', 'sinc256', 'sinc64', 'spline16', 'spline36'
         """
@@ -264,10 +266,10 @@ ACCEPTS: numeric/numarray/PIL Image A"""
 
     def set_aspect(self, s):
         """
-Set the method used to constrain the aspoect ratio of the
-image ehen resizing,
+        Set the method used to constrain the aspoect ratio of the
+        image ehen resizing,
 
-ACCEPTS ['free' | 'preserve']"""
+        ACCEPTS ['free' | 'preserve']"""
 
         s = s.lower()
         if not self._aspectd.has_key(s):
@@ -277,20 +279,21 @@ ACCEPTS ['free' | 'preserve']"""
 
     def set_interpolation(self, s):
         """
-Set the interpolation method the image uses when resizing.
+        Set the interpolation method the image uses when resizing.
+
+        ACCEPTS: ['bicubic' | 'bilinear' | 'blackman100' | 'blackman256' | 'blackman64', 'nearest' | 'sinc144' | 'sinc256' | 'sinc64' | 'spline16' | 'spline36']
+        """
 
-ACCEPTS: ['bicubic' | 'bilinear' | 'blackman100' | 'blackman256' | 'blackman64', 'nearest' | 'sinc144' | 'sinc256' | 'sinc64' | 'spline16' | 'spline36']"""
-        
         s = s.lower()
         if not self._interpd.has_key(s):
             raise ValueError('Illegal interpolation string')
         self._interpolation = s
-        
+
     def get_extent(self):
         'get the image extent: xmin, xmax, ymin, ymax'
         if self._extent is not None:
             return self._extent
-        else:            
+        else:
             numrows, numcols = self.get_size()
             iwidth, iheight = numcols, numrows
             #return 0, width, 0, height
@@ -298,13 +301,13 @@ ACCEPTS: ['bicubic' | 'bilinear' | 'blackman100' | 'blackman256' | 'blackman64',
             sx = dwidth  / iwidth
             sy = dheight / iheight
 
-            if self.get_aspect()=='preserve' and sy<sx: sx = sy 
+            if self.get_aspect()=='preserve' and sy<sx: sx = sy
             return 0, 1.0/sx*dwidth, 0, 1.0/sy*dheight
 
     def set_filternorm(self, filternorm):
         """Set whether the resize filter norms the weights -- see help for imshow
-ACCEPTS: 0 or 1
-"""
+        ACCEPTS: 0 or 1
+        """
         if filternorm:
             self._filternorm = 1
         else:
@@ -316,17 +319,110 @@ ACCEPTS: 0 or 1
 
     def set_filterrad(self, filterrad):
         """Set the resize filter radius only applicable to some
-interpolation schemes -- see help for imshow
+        interpolation schemes -- see help for imshow
 
-ACCEPTS: positive float
-"""
+        ACCEPTS: positive float
+        """
         r = float(filterrad)
         assert(r>0)
         self._filterrad = r
 
     def get_filterrad(self):
         'return the filterrad setting'
-        return self._filterrad
+
+
+class NonUniformImage(AxesImage):
+    def __init__(self, ax,
+                 cmap = None,
+                 norm = None,
+                 extent=None,
+                ):
+        AxesImage.__init__(self, ax,
+                           cmap = cmap,
+                           norm = norm,
+                           extent=extent,
+                           aspect = 'free',
+                           interpolation = 'nearest',
+                           origin = 'lower',
+                          )
+
+    def make_image(self):
+        if self._A is None:
+            raise RuntimeError('You must first set the image array')
+
+        x0, y0, v_width, v_height = self.axes.viewLim.get_bounds()
+        l, b, width, height = self.axes.bbox.get_bounds()
+        im = _image.pcolor(self._Ax, self._Ay, self._A,
+                           height, width,
+                           (x0, x0+v_width, y0, y0+v_height),
+                          )
+
+        bg = colorConverter.to_rgba(self.axes.get_frame().get_facecolor(), 0)
+        im.set_bg(*bg)
+        im.set_aspect(self._aspectd[self._aspect])
+        return im
+
+    def set_data(self, x, y, A):
+        x = asarray(x).astype(Float32)
+        y = asarray(y).astype(Float32)
+        A = asarray(A)
+        if len(x.shape) != 1 or len(y.shape) != 1\
+           or A.shape[0:2] != (y.shape[0], x.shape[0]):
+            raise TypeError("Axes don't match array shape")
+        if len(A.shape) not in [2, 3]:
+            raise TypeError("Can only plot 2D or 3D data")
+        if len(A.shape) == 3 and A.shape[2] not in [1, 3, 4]:
+            raise TypeError("3D arrays must have three (RGB) or four (RGBA) color components")
+        if len(A.shape) == 3 and A.shape[2] == 1:
+             A.shape = A.shape[0:2]
+        if len(A.shape) == 2:
+            if typecode(A) != UInt8:
+                A = (self.cmap(self.norm(A))*255).astype(UInt8)
+            else:
+                A = repeat(A[:,:,NewAxis], 4, 2)
+                A[:,:,3] = 255
+        else:
+            if typecode(A) != UInt8:
+                A = (255*A).astype(UInt8)
+            if A.shape[2] == 3:
+                B = zeros(tuple(list(A.shape[0:2]) + [4]), UInt8)
+                B[:,:,0:3] = A
+                B[:,:,3] = 255
+                A = B
+        self._A = A
+        self._Ax = x
+        self._Ay = y
+        self._imcache = None
+
+    def set_array(self, *args):
+        raise NotImplementedError('Method not supported')
+
+    def set_interpolation(self, s):
+        if s != 'nearest':
+            raise NotImplementedError('Only nearest neighbor supported')
+
+    def get_extent(self):
+        if self._A is None:
+            raise RuntimeError('Must set data first')
+        return self._Ax[0], self._Ax[-1], self._Ay[0], self._Ay[-1]
+
+    def set_filternorm(self, s):
+        pass
+
+    def set_filterrad(self, s):
+        pass
+
+    def set_norm(self, norm):
+        if self._A is not None:
+            raise RuntimeError('Cannot change colors after loading data')
+        cm.ScalarMappable.set_norm(self, norm)
+
+    def set_cmap(self, cmap):
+        if self._A is not None:
+            raise RuntimeError('Cannot change colors after loading data')
+        cm.ScalarMappable.set_cmap(self, norm)
+
+
 
 
 class FigureImage(Artist, cm.ScalarMappable):
@@ -341,7 +437,7 @@ class FigureImage(Artist, cm.ScalarMappable):
         """
         cmap is a cm colormap instance
         norm is a colors.normalize instance to map luminance to 0-1
-        
+
         """
         Artist.__init__(self)
         cm.ScalarMappable.__init__(self, norm, cmap)
@@ -350,7 +446,7 @@ class FigureImage(Artist, cm.ScalarMappable):
         self.figure = fig
         self.ox = offsetx
         self.oy = offsety
-        
+
 
     def get_size(self):
         'Get the numrows, numcols of the input image'
@@ -359,20 +455,20 @@ class FigureImage(Artist, cm.ScalarMappable):
 
         return self._A.shape[:2]
 
-    def make_image(self):        
+    def make_image(self):
         if self._A is None:
             raise RuntimeError('You must first set the image array')
-        
+
         x = self.to_rgba(self._A, self._alpha)
 
         im = _image.fromarray(x, 1)
-        im.set_bg( *colorConverter.to_rgba(self.figure.get_facecolor(), 0) )        
+        im.set_bg( *colorConverter.to_rgba(self.figure.get_facecolor(), 0) )
         im.is_grayscale = (self.cmap.name == "gray" and
                            len(self._A.shape) == 2)
         return im
-    
+
     def draw(self, renderer, *args, **kwargs):
-        if not self.get_visible(): return 
+        if not self.get_visible(): return
         im = self.make_image()
         renderer.draw_image(self.ox, self.oy, im, self.origin, self.figure.bbox)
 
@@ -398,7 +494,7 @@ def imread(fname):
     handler = handlers[ext]
     return handler(fname)
 
-    
+
 
 def pil_to_array( pilImage ):
     if pilImage.mode in ('RGBA', 'RGBX'):
