@@ -13,10 +13,10 @@ FT2Image::~FT2Image() {delete [] buffer; buffer=NULL;}
 Glyph::Glyph( const FT_Face& face, const FT_Glyph& glyph, size_t ind) :
   glyphInd(ind) {
   _VERBOSE("Glyph::Glyph");
-  
+
   FT_BBox bbox;
   FT_Glyph_Get_CBox( glyph, ft_glyph_bbox_subpixels, &bbox );
-  
+
   setattr("width",        Py::Int( face->glyph->metrics.width) );
   setattr("height",       Py::Int( face->glyph->metrics.height) );
   setattr("horiBearingX", Py::Int( face->glyph->metrics.horiBearingX) );
@@ -24,14 +24,14 @@ Glyph::Glyph( const FT_Face& face, const FT_Glyph& glyph, size_t ind) :
   setattr("horiAdvance",  Py::Int( face->glyph->metrics.horiAdvance) );
   setattr("linearHoriAdvance",  Py::Int( face->glyph->linearHoriAdvance) );
   setattr("vertBearingX", Py::Int( face->glyph->metrics.vertBearingX) );
-  
+
   setattr("vertBearingY", Py::Int( face->glyph->metrics.vertBearingY) );
   setattr("vertAdvance",  Py::Int( face->glyph->metrics.vertAdvance) );
   //setattr("bitmap_left",  Py::Int( face->glyph->bitmap_left) );
   //setattr("bitmap_top",  Py::Int( face->glyph->bitmap_top) );
-  
+
   Py::Tuple abbox(4);
-  
+
   abbox[0] = Py::Int(bbox.xMin);
   abbox[1] = Py::Int(bbox.yMin);
   abbox[2] = Py::Int(bbox.xMax);
@@ -44,11 +44,11 @@ Glyph::~Glyph() {
   _VERBOSE("Glyph::~Glyph");
 }
 
-int 
+int
 Glyph::setattr( const char *name, const Py::Object &value ) {
   _VERBOSE("Glyph::setattr");
   __dict__[name] = value;
-  return 0; 
+  return 0;
 }
 
 Py::Object
@@ -76,40 +76,40 @@ Glyph::get_path( const FT_Face& face) {
   Py::List path;
   bool flip_y= false; //todo, pass me as kwarg
 
- 
+
   FT_Vector   v_last;
   FT_Vector   v_control;
   FT_Vector   v_start;
-  
+
   FT_Vector*  point;
   FT_Vector*  limit;
   char*       tags;
-  
+
   int   n;         // index of contour in outline
   int   first;     // index of first point in contour
   char  tag;       // current point's state
-  
+
   first = 0;
-  
+
   for(n = 0; n < outline.n_contours; n++)
     {
       int  last;  // index of last point in contour
-      
+
       last  = outline.contours[n];
       limit = outline.points + last;
-      
+
       v_start = outline.points[first];
       v_last  = outline.points[last];
-      
+
       v_control = v_start;
-      
+
       point = outline.points + first;
       tags  = outline.tags  + first;
       tag   = FT_CURVE_TAG(tags[0]);
 
       // A contour cannot start with a cubic control point!
       if(tag == FT_CURVE_TAG_CUBIC) return Py::Object();
-      
+
       // check first point to determine origin
       if( tag == FT_CURVE_TAG_CONIC)
 	{
@@ -127,13 +127,13 @@ Glyph::get_path( const FT_Face& face) {
 	      // for closure
 	      v_start.x = (v_start.x + v_last.x) / 2;
 	      v_start.y = (v_start.y + v_last.y) / 2;
-	      
+
 	      v_last = v_start;
 	    }
 	  point--;
 	  tags--;
 	}
-      
+
       double x = conv(v_start.x);
       double y = flip_y ? -conv(v_start.y) : conv(v_start.y);
       Py::Tuple tup(3);
@@ -145,12 +145,12 @@ Glyph::get_path( const FT_Face& face) {
       Py::Tuple closepoly(2);
       closepoly[0] = Py::Int(ENDPOLY);
       closepoly[1] = Py::Int(0);
-      
+
       while(point < limit)
 	{
 	  point++;
 	  tags++;
-	  
+
 	  tag = FT_CURVE_TAG(tags[0]);
 	  switch(tag)
 	    {
@@ -166,25 +166,25 @@ Glyph::get_path( const FT_Face& face) {
 
 		continue;
 	      }
-	      
+
 	    case FT_CURVE_TAG_CONIC:  // consume conic arcs
 	      {
 		v_control.x = point->x;
 		v_control.y = point->y;
-		
+
 	      Do_Conic:
 		if(point < limit)
 		  {
 		    FT_Vector vec;
 		    FT_Vector v_middle;
-		    
+
 		    point++;
 		    tags++;
 		    tag = FT_CURVE_TAG(tags[0]);
-		    
+
 		    vec.x = point->x;
 		    vec.y = point->y;
-		    
+
 		    if(tag == FT_CURVE_TAG_ON)
 		      {
 			double xctl = conv(v_control.x);
@@ -200,12 +200,12 @@ Glyph::get_path( const FT_Face& face) {
 			path.append(tup);
 			continue;
 		      }
-		    
+
 		    if(tag != FT_CURVE_TAG_CONIC) return Py::Object();
-		    
+
 		    v_middle.x = (v_control.x + vec.x) / 2;
 		    v_middle.y = (v_control.y + vec.y) / 2;
-		    
+
 		    double xctl = conv(v_control.x);
 		    double yctl = flip_y ? -conv(v_control.y) : conv(v_control.y);
 		    double xto = conv(v_middle.x);
@@ -234,28 +234,28 @@ Glyph::get_path( const FT_Face& face) {
 		path.append(tup);
 		goto Close;
 	      }
-	      
+
 	    default:  // FT_CURVE_TAG_CUBIC
 	      {
 		FT_Vector vec1, vec2;
-		
+
 		if(point + 1 > limit || FT_CURVE_TAG(tags[1]) != FT_CURVE_TAG_CUBIC)
 		  {
 		    return Py::Object();
 		  }
-		
-		vec1.x = point[0].x; 
+
+		vec1.x = point[0].x;
 		vec1.y = point[0].y;
-		vec2.x = point[1].x; 
+		vec2.x = point[1].x;
 		vec2.y = point[1].y;
-		
+
 		point += 2;
 		tags  += 2;
-		
+
 		if(point <= limit)
 		  {
 		    FT_Vector vec;
-		    
+
 		    vec.x = point->x;
 		    vec.y = point->y;
 
@@ -274,7 +274,7 @@ Glyph::get_path( const FT_Face& face) {
 		    tup[5] = Py::Float(xto);
 		    tup[6] = Py::Float(yto);
 		    path.append(tup);
-		    
+
 		    continue;
 		  }
 
@@ -293,31 +293,31 @@ Glyph::get_path( const FT_Face& face) {
 		tup[5] = Py::Float(xto);
 		tup[6] = Py::Float(yto);
 		path.append(tup);
-		
+
 		goto Close;
 	      }
 	    }
 	}
 
       path.append(closepoly);
-		
-      
+
+
     Close:
-      first = last + 1; 
+      first = last + 1;
     }
-  
+
   return path;
 }
 
 
-FT2Font::FT2Font(std::string facefile) 
+FT2Font::FT2Font(std::string facefile)
 {
-  _VERBOSE("FT2Font::FT2Font");
+  _VERBOSE(Printf("FT2Font::FT2Font %s", facefile.c_str()).str());
   clear(Py::Tuple(0));
 
-  
+
   int error = FT_New_Face( _ft2Library, facefile.c_str(), 0, &face );
-  
+
 
   if (error == FT_Err_Unknown_File_Format ) {
     std::ostringstream s;
@@ -339,7 +339,7 @@ FT2Font::FT2Font(std::string facefile)
     s << "Could not open facefile " << facefile << "; freetype error code " << error<< std::endl;
     throw Py::RuntimeError(s.str());
   }
-  
+
   // set a default fontsize 12 pt at 72dpi
   error = FT_Set_Char_Size( face, 12 * 64, 0, 72, 72 );
   //error = FT_Set_Char_Size( face, 20 * 64, 0, 80, 80 );
@@ -348,15 +348,15 @@ FT2Font::FT2Font(std::string facefile)
     s << "Could not set the fontsize for facefile  " << facefile << std::endl;
     throw Py::RuntimeError(s.str());
   }
-  
+
   // set some face props as attributes
-  //small memory leak fixed after 2.1.8 
+  //small memory leak fixed after 2.1.8
   //fields can be null so we have to check this first
 
   const char* ps_name = FT_Get_Postscript_Name( face );
   if ( ps_name == NULL )
     ps_name = "UNAVAILABLE";
-  
+
    const char* family_name = face->family_name;
    if ( family_name == NULL )
      family_name = "UNAVAILABLE";
@@ -375,14 +375,14 @@ FT2Font::FT2Font(std::string facefile)
   setattr("num_glyphs",      Py::Int(face->num_glyphs));
   setattr("num_fixed_sizes", Py::Int(face->num_fixed_sizes));
   setattr("num_charmaps",    Py::Int(face->num_charmaps));
-  
+
   int scalable = FT_IS_SCALABLE( face );
-  
+
   setattr("scalable", Py::Int(scalable));
-  
+
   if (scalable) {
     setattr("units_per_EM", Py::Int(face->units_per_EM));
-    
+
     Py::Tuple bbox(4);
     bbox[0] = Py::Int(face->bbox.xMin);
     bbox[1] = Py::Int(face->bbox.yMin);
@@ -397,18 +397,20 @@ FT2Font::FT2Font(std::string facefile)
     setattr("underline_position",  Py::Int(face->underline_position));
     setattr("underline_thickness", Py::Int(face->underline_thickness));
   }
+
+  _VERBOSE("FT2Font::FT2Font done");
 }
 
 FT2Font::~FT2Font()
 {
   _VERBOSE("FT2Font::~FT2Font");
   FT_Done_Face    ( face );
-  
+
   delete [] image.buffer ;
   image.buffer = NULL;
-  
+
   for (size_t i=0; i<glyphs.size(); i++) {
-    FT_Done_Glyph( glyphs[i] );    
+    FT_Done_Glyph( glyphs[i] );
   }
   for (size_t i=0; i<gms.size(); i++) {
     Py_DECREF(gms[i]);
@@ -422,70 +424,70 @@ char  FT2Font::horiz_image_to_vert_image__doc__[] =
 "Copies the horizontal image (w, h) into a\n"
 "new image of size (h,w)\n"
 "This is equivalent to rotating the original image\n"
-"by 90 degrees ccw\n" 
+"by 90 degrees ccw\n"
 ;
 
 Py::Object
 FT2Font::horiz_image_to_vert_image(const Py::Tuple & args) {
-  
+
   // If we have already rotated, just return.
-  
-  if (image.bRotated) 
-    return Py::Object(); 
-  
-  
+
+  if (image.bRotated)
+    return Py::Object();
+
+
   long width = image.width, height = image.height;
-  
+
   long newWidth  = image.height;
   long newHeight = image.width;
-  
+
   long numBytes = image.width * image.height;
-  
+
   unsigned char * buffer = new unsigned char [numBytes];
-  
-  long  i, j, k, offset, nhMinusOne;   
-  
+
+  long  i, j, k, offset, nhMinusOne;
+
   nhMinusOne = newHeight-1;
-  
+
   for (i=0; i<height; i++) {
-    
+
     offset = i*width;
-    
+
     for (j=0; j<width; j++) {
-      
-      k = nhMinusOne - j; 
-      
+
+      k = nhMinusOne - j;
+
       buffer[i + k*newWidth] = image.buffer[j + offset];
-      
+
     }
-    
+
   }
-  
+
   delete [] image.buffer;
   image.buffer = buffer;
   image.width = newWidth;
   image.height = newHeight;
   image.bRotated = true;
-  
+
   return Py::Object();
-  
+
 }
 
-int 
+int
 FT2Font::setattr( const char *name, const Py::Object &value ) {
   _VERBOSE("FT2Font::setattr");
   __dict__[name] = value;
-  return 1; 
+  return 1;
 }
 
-Py::Object 
+Py::Object
 FT2Font::getattr( const char *name ) {
   _VERBOSE("FT2Font::getattr");
   if ( __dict__.hasKey(name) ) return __dict__[name];
   else return getattr_default( name );
 }
 
-char FT2Font::set_bitmap_size__doc__[] = 
+char FT2Font::set_bitmap_size__doc__[] =
 "set_bitmap_size(w, h)\n"
 "\n"
 "Manually set the bitmap size to render the glyps to.  This is useful"
@@ -496,24 +498,24 @@ Py::Object
 FT2Font::set_bitmap_size(const Py::Tuple & args) {
   _VERBOSE("FT2Font::set_bitmap_size");
   args.verify_length(2);
-  
+
   long width = Py::Int(args[0]);
   long height = Py::Int(args[1]);
-  
+
   image.width   = (unsigned)width;
   image.height  = (unsigned)height;
-  
+
   long numBytes = image.width * image.height;
-  
+
   delete [] image.buffer;
   image.buffer = new unsigned char [numBytes];
-  for (long n=0; n<numBytes; n++) 
+  for (long n=0; n<numBytes; n++)
     image.buffer[n] = 0;
-  
+
   return Py::Object();
 }
 
-char FT2Font::clear__doc__[] = 
+char FT2Font::clear__doc__[] =
 "clear()\n"
 "\n"
 "Clear all the glyphs, reset for a new set_text"
@@ -523,7 +525,7 @@ Py::Object
 FT2Font::clear(const Py::Tuple & args) {
   _VERBOSE("FT2Font::clear");
   args.verify_length(0);
-  
+
   //todo: move to image method?
   delete [] image.buffer ;
   image.buffer  = NULL;
@@ -531,30 +533,30 @@ FT2Font::clear(const Py::Tuple & args) {
   image.height  = 0;
   image.offsetx = 0;
   image.offsety = 0;
-  
+
   angle = 0.0;
-  
+
   pen.x = 0;
   pen.y = 0;
-  
+
   for (size_t i=0; i<glyphs.size(); i++) {
-    FT_Done_Glyph( glyphs[i] );    
+    FT_Done_Glyph( glyphs[i] );
   }
-  
+
   for (size_t i=0; i<gms.size(); i++) {
     Py_DECREF(gms[i]);
   }
-  
+
   glyphs.resize(0);
   gms.resize(0);
-  
+
   return Py::Object();
 }
 
 
 
 
-char FT2Font::set_size__doc__[] = 
+char FT2Font::set_size__doc__[] =
 "set_size(ptsize, dpi)\n"
 "\n"
 "Set the point size and dpi of the text.\n"
@@ -564,20 +566,20 @@ Py::Object
 FT2Font::set_size(const Py::Tuple & args) {
   _VERBOSE("FT2Font::set_size");
   args.verify_length(2);
-  
+
   double ptsize = Py::Float(args[0]);
   double dpi = Py::Float(args[1]);
-  
-  int error = FT_Set_Char_Size( face, (long)(ptsize * 64), 0, 
-				(unsigned int)dpi, 
+
+  int error = FT_Set_Char_Size( face, (long)(ptsize * 64), 0,
+				(unsigned int)dpi,
 				(unsigned int)dpi );
-  if (error) 
-    throw Py::RuntimeError("Could not set the fontsize");  
+  if (error)
+    throw Py::RuntimeError("Could not set the fontsize");
   return Py::Object();
 }
 
 
-char FT2Font::set_charmap__doc__[] = 
+char FT2Font::set_charmap__doc__[] =
 "set_charmap(i)\n"
 "\n"
 "Make the i-th charmap current\n"
@@ -587,9 +589,9 @@ Py::Object
 FT2Font::set_charmap(const Py::Tuple & args) {
   _VERBOSE("FT2Font::set_charmap");
   args.verify_length(1);
-  
+
   int i = Py::Int(args[0]);
-  if (i>=face->num_charmaps) 
+  if (i>=face->num_charmaps)
     throw Py::ValueError("i exceeds the available number of char maps");
   FT_CharMap charmap = face->charmaps[i];
   if (FT_Set_Charmap( face, charmap ))
@@ -600,12 +602,12 @@ FT2Font::set_charmap(const Py::Tuple & args) {
 FT_BBox
 FT2Font::compute_string_bbox(  ) {
   _VERBOSE("FT2Font::compute_string_bbox");
-  
+
   FT_BBox bbox;
   /* initialize string bbox to "empty" values */
   bbox.xMin = bbox.yMin = 32000;
   bbox.xMax = bbox.yMax = -32000;
-  
+
   for ( size_t n = 0; n < glyphs.size(); n++ ) {
     FT_BBox glyph_bbox;
     FT_Glyph_Get_CBox( glyphs[n], ft_glyph_bbox_subpixels, &glyph_bbox );
@@ -622,7 +624,7 @@ FT2Font::compute_string_bbox(  ) {
 }
 
 
-char FT2Font::get_kerning__doc__[] = 
+char FT2Font::get_kerning__doc__[] =
 "dx = get_kerning(left, right, mode)\n"
 "\n"
 "Get the kerning between left char and right glyph indices\n"
@@ -638,12 +640,12 @@ FT2Font::get_kerning(const Py::Tuple & args) {
   int left = Py::Int(args[0]);
   int right = Py::Int(args[1]);
   int mode = Py::Int(args[2]);
-  
+
 
   if (!FT_HAS_KERNING( face )) return Py::Int(0);
   FT_Vector delta;
-   
-  if (!FT_Get_Kerning( face, left, right, mode, &delta )) {    
+
+  if (!FT_Get_Kerning( face, left, right, mode, &delta )) {
     return Py::Int(delta.x);
   }
   else {
@@ -654,7 +656,7 @@ FT2Font::get_kerning(const Py::Tuple & args) {
 
 
 
-char FT2Font::set_text__doc__[] = 
+char FT2Font::set_text__doc__[] =
 "set_text(s, angle)\n"
 "\n"
 "Set the text string and angle.\n"
@@ -664,10 +666,10 @@ Py::Object
 FT2Font::set_text(const Py::Tuple & args) {
   _VERBOSE("FT2Font::set_text");
   args.verify_length(2);
-  
+
 
   Py::String text( args[0] );
-  std::string stdtext;
+  std::string stdtext="";
   Py_UNICODE* pcode=NULL;
   size_t N = 0;
   if (PyUnicode_Check(text.ptr())) {
@@ -678,8 +680,8 @@ FT2Font::set_text(const Py::Tuple & args) {
     stdtext = text.as_std_string();
     N = stdtext.size();
   }
-  
-  
+
+
   angle = Py::Float(args[1]);
 
   angle = angle/360.0*2*3.14159;
@@ -688,20 +690,20 @@ FT2Font::set_text(const Py::Tuple & args) {
   matrix.xy = (FT_Fixed)(-sin( angle ) * 0x10000L );
   matrix.yx = (FT_Fixed)( sin( angle ) * 0x10000L );
   matrix.yy = (FT_Fixed)( cos( angle ) * 0x10000L );
-  
-  FT_Bool use_kerning = FT_HAS_KERNING( face ); 
-  FT_UInt previous = 0; 
-  
+
+  FT_Bool use_kerning = FT_HAS_KERNING( face );
+  FT_UInt previous = 0;
+
   glyphs.resize(0);
   pen.x = 0;
   pen.y = 0;
 
   Py::Tuple xys(N);
-  for ( unsigned int n = 0; n < N; n++ ) { 
+  for ( unsigned int n = 0; n < N; n++ ) {
     std::string thischar("?");
     FT_UInt glyph_index;
 
-    
+
     if (pcode==NULL) {
       // plain ol string
       thischar = stdtext[n];
@@ -712,52 +714,52 @@ FT2Font::set_text(const Py::Tuple & args) {
       glyph_index = FT_Get_Char_Index( face, pcode[n] );
     }
 
-    // retrieve kerning distance and move pen position 
-    if ( use_kerning && previous && glyph_index ) { 
+    // retrieve kerning distance and move pen position
+    if ( use_kerning && previous && glyph_index ) {
       FT_Vector delta;
       FT_Get_Kerning( face, previous, glyph_index,
 		      FT_KERNING_DEFAULT, &delta );
       pen.x += delta.x;
     }
-    error = FT_Load_Glyph( face, glyph_index, FT_LOAD_DEFAULT ); 
+    error = FT_Load_Glyph( face, glyph_index, FT_LOAD_DEFAULT );
     if ( error ) {
       std::cerr << "\tcould not load glyph for " << thischar << std::endl;
-      continue; 
+      continue;
     }
-    // ignore errors, jump to next glyph 
-    
+    // ignore errors, jump to next glyph
+
     // extract glyph image and store it in our table
-    
-    FT_Glyph thisGlyph; 
-    error = FT_Get_Glyph( face->glyph, &thisGlyph ); 
-    
+
+    FT_Glyph thisGlyph;
+    error = FT_Get_Glyph( face->glyph, &thisGlyph );
+
     if ( error ) {
       std::cerr << "\tcould not get glyph for " << thischar << std::endl;
-      continue; 
+      continue;
     }
-    // ignore errors, jump to next glyph 
-    
+    // ignore errors, jump to next glyph
+
     FT_Glyph_Transform( thisGlyph, 0, &pen);
     Py::Tuple xy(2);
     xy[0] = Py::Float(pen.x);
     xy[1] = Py::Float(pen.y);
     xys[n] = xy;
     pen.x += face->glyph->advance.x;
-    
-    previous = glyph_index; 
+
+    previous = glyph_index;
     glyphs.push_back(thisGlyph);
   }
 
   // now apply the rotation
-  for (unsigned int n=0; n<glyphs.size(); n++) 
+  for (unsigned int n=0; n<glyphs.size(); n++)
     FT_Glyph_Transform(glyphs[n], &matrix, 0);
 
-  
+  _VERBOSE("FT2Font::set_text done");
   return xys;
 }
 
 
-char FT2Font::get_glyph__doc__[] = 
+char FT2Font::get_glyph__doc__[] =
 "get_glyph(num)\n"
 "\n"
 "Return the glyph object with num num\n"
@@ -765,18 +767,18 @@ char FT2Font::get_glyph__doc__[] =
 Py::Object
 FT2Font::get_glyph(const Py::Tuple & args){
   _VERBOSE("FT2Font::get_glyph");
-  
+
   args.verify_length(1);
   int num = Py::Int(args[0]);
-  
-  if ( (size_t)num >= gms.size()) 
+
+  if ( (size_t)num >= gms.size())
     throw Py::ValueError("Glyph index out of range");
-  
+
   //todo: refcount?
   return Py::asObject(gms[num]);
 }
 
-char FT2Font::get_num_glyphs__doc__[] = 
+char FT2Font::get_num_glyphs__doc__[] =
 "get_num_glyphs()\n"
 "\n"
 "Return the number of loaded glyphs\n"
@@ -785,18 +787,18 @@ Py::Object
 FT2Font::get_num_glyphs(const Py::Tuple & args){
   _VERBOSE("FT2Font::get_num_glyphs");
   args.verify_length(0);
-  
+
   return Py::Int((long)glyphs.size());
 }
 
-char FT2Font::load_char__doc__[] = 
+char FT2Font::load_char__doc__[] =
 "load_char(charcode)\n"
 "\n"
 "Load character with charcode in current fontfile and set glyph.\n"
 "Return value is a Glyph object, with attributes\n"
 "  width          # glyph width\n"
 "  height         # glyph height\n"
-"  bbox           # the glyph bbox (xmin, ymin, xmax, ymax)\n" 
+"  bbox           # the glyph bbox (xmin, ymin, xmax, ymax)\n"
 "  horiBearingX   # left side bearing in horizontal layouts\n"
 "  horiBearingY   # top side bearing in horizontal layouts\n"
 "  horiAdvance    # advance width for horizontal layout\n"
@@ -810,20 +812,20 @@ FT2Font::load_char(const Py::Tuple & args) {
   //load a char using the unsigned long charcode
   args.verify_length(1);
   long charcode = Py::Int(args[0]);
-  
+
   int error = FT_Load_Char( face, (unsigned long)charcode, FT_LOAD_DEFAULT);
   //int error = FT_Load_Char( face, (unsigned long)charcode, FT_LOAD_LINEAR_DESIGN);
-  //int error = FT_Load_Char( face, (unsigned long)charcode, FT_LOAD_RENDER); 
-  
+  //int error = FT_Load_Char( face, (unsigned long)charcode, FT_LOAD_RENDER);
+
   if (error)
     throw Py::RuntimeError(Printf("Could not load charcode %d", charcode).str());
-  
+
   FT_Glyph thisGlyph;
   error = FT_Get_Glyph( face->glyph, &thisGlyph );
-  
+
   if (error)
     throw Py::RuntimeError(Printf("Could not get glyph for char %d", charcode).str());
-  
+
   size_t num = glyphs.size();  //the index into the glyphs list
   glyphs.push_back(thisGlyph);
   Glyph* gm = new Glyph(face, thisGlyph, num);
@@ -832,7 +834,7 @@ FT2Font::load_char(const Py::Tuple & args) {
   return Py::asObject( gm);
 }
 
-char FT2Font::get_width_height__doc__[] = 
+char FT2Font::get_width_height__doc__[] =
 "w, h = get_width_height()\n"
 "\n"
 "Get the width and height in 26.6 subpixels of the current string set by set_text\n"
@@ -843,18 +845,18 @@ Py::Object
 FT2Font::get_width_height(const Py::Tuple & args) {
   _VERBOSE("FT2Font::get_width_height");
   args.verify_length(0);
-  
+
 
 
   FT_BBox bbox = compute_string_bbox();
-  
+
   Py::Tuple ret(2);
   ret[0] = Py::Int(bbox.xMax - bbox.xMin);
   ret[1] = Py::Int(bbox.yMax - bbox.yMin);
   return ret;
 }
 
-char FT2Font::get_descent__doc__[] = 
+char FT2Font::get_descent__doc__[] =
 "d = get_descent()\n"
 "\n"
 "Get the descent of the current string set by set_text in 26.6 subpixels.\n"
@@ -865,7 +867,7 @@ Py::Object
 FT2Font::get_descent(const Py::Tuple & args) {
   _VERBOSE("FT2Font::get_descent");
   args.verify_length(0);
-  
+
   FT_BBox bbox = compute_string_bbox();
   return Py::Int(- bbox.yMin);;
 }
@@ -878,7 +880,7 @@ FT2Font::draw_bitmap( FT_Bitmap*  bitmap,
   FT_Int  i, j, p, q;
   FT_Int  x_max = x + bitmap->width;
   FT_Int  y_max = y + bitmap->rows;
-  
+
   FT_Int width = (FT_Int)image.width;
   FT_Int height = (FT_Int)image.height;
   for ( i = x, p = 0; i < x_max; i++, p++ )
@@ -888,12 +890,12 @@ FT2Font::draw_bitmap( FT_Bitmap*  bitmap,
 	  if ( i >= width || j >= height )
 	    continue;
 	  image.buffer[i + j*width] |= bitmap->buffer[p + q*bitmap->width];
-	  
+
 	}
     }
 }
 
-char FT2Font::write_bitmap__doc__[] = 
+char FT2Font::write_bitmap__doc__[] =
 "write_bitmap(fname)\n"
 "\n"
 "Write the bitmap to file fname\n"
@@ -901,27 +903,27 @@ char FT2Font::write_bitmap__doc__[] =
 Py::Object
 FT2Font::write_bitmap(const Py::Tuple & args) {
   _VERBOSE("FT2Font::write_bitmap");
-  
+
   args.verify_length(1);
-  
+
   FT_Int  i, j;
-  
+
   std::string filename = Py::String(args[0]);
-  
+
   FILE *fh = fopen(filename.c_str(), "w");
   FT_Int width = (FT_Int)image.width;
   FT_Int height = (FT_Int)image.height;
-  
-  for ( i = 0; i< height; i++)   
-    for ( j = 0; j < width; ++j) 
+
+  for ( i = 0; i< height; i++)
+    for ( j = 0; j < width; ++j)
       fputc(image.buffer[j + i*width], fh);
-  
+
   fclose(fh);
-  
+
   return Py::Object();
 }
 
-char FT2Font::draw_rect__doc__[] = 
+char FT2Font::draw_rect__doc__[] =
 "draw_bbox(x0, y0, x1, y1)\n"
 "\n"
 "Draw a rect to the image.  It is your responsibility to set the dimensions\n"
@@ -931,27 +933,27 @@ char FT2Font::draw_rect__doc__[] =
 Py::Object
 FT2Font::draw_rect(const Py::Tuple & args) {
   _VERBOSE("FT2Font::draw_rect");
-  
+
   args.verify_length(4);
-  
+
   long x0 = Py::Int(args[0]);
   long y0 = Py::Int(args[1]);
   long x1 = Py::Int(args[2]);
   long y1 = Py::Int(args[3]);
-  
+
   FT_Int iwidth = (FT_Int)image.width;
   FT_Int iheight = (FT_Int)image.height;
-  
-  if ( x0<0 || y0<0 || x1<0 || y1<0 || 
+
+  if ( x0<0 || y0<0 || x1<0 || y1<0 ||
        x0>iwidth || x1>iwidth ||
-       y0>iheight || y1>iheight ) 
+       y0>iheight || y1>iheight )
     throw Py::ValueError("Rect coords outside image bounds");
-  
+
   for (long i=x0; i<x1; ++i) {
     image.buffer[i + y0*iwidth] = 255;
     image.buffer[i + y1*iwidth] = 255;
   }
-  
+
   for (long j=y0; j<y1; ++j) {
     image.buffer[x0 + j*iwidth] = 255;
     image.buffer[x1 + j*iwidth] = 255;
@@ -959,7 +961,7 @@ FT2Font::draw_rect(const Py::Tuple & args) {
   return Py::Object();
 }
 
-char FT2Font::image_as_str__doc__[] = 
+char FT2Font::image_as_str__doc__[] =
 "width, height, s = image_as_str()\n"
 "\n"
 "Return the image buffer as a string\n"
@@ -969,17 +971,17 @@ Py::Object
 FT2Font::image_as_str(const Py::Tuple & args) {
   _VERBOSE("FT2Font::image_as_str");
   args.verify_length(0);
-  
+
   return Py::asObject(
-		      Py_BuildValue("lls#", 
-				    image.width, 
-				    image.height, 
-				    image.buffer, 
+		      Py_BuildValue("lls#",
+				    image.width,
+				    image.height,
+				    image.buffer,
 				    image.width*image.height)
 		      );
 }
 
-char FT2Font::draw_glyphs_to_bitmap__doc__[] = 
+char FT2Font::draw_glyphs_to_bitmap__doc__[] =
 "draw_glyphs_to_bitmap()\n"
 "\n"
 "Draw the glyphs that were loaded by set_text to the bitmap\n"
@@ -987,72 +989,72 @@ char FT2Font::draw_glyphs_to_bitmap__doc__[] =
 ;
 Py::Object
 FT2Font::draw_glyphs_to_bitmap(const Py::Tuple & args) {
-  
+
   _VERBOSE("FT2Font::draw_glyphs_to_bitmap");
   args.verify_length(0);
-  
+
   FT_BBox string_bbox = compute_string_bbox();
-  
+
   image.width   = (string_bbox.xMax-string_bbox.xMin) / 64+2;
   image.height  = (string_bbox.yMax-string_bbox.yMin) / 64+2;
-  
+
   image.offsetx = (int)(string_bbox.xMin/64.0);
-  if (angle==0)  
+  if (angle==0)
     image.offsety = -image.height;
   else {
     image.offsety = (int)(-string_bbox.yMax/64.0);
   }
-  
+
   size_t numBytes = image.width*image.height;
   delete [] image.buffer;
   image.buffer = new unsigned char [numBytes];
-  for (size_t n=0; n<numBytes; n++) 
+  for (size_t n=0; n<numBytes; n++)
     image.buffer[n] = 0;
-  
-  
-  
+
+
+
   for ( size_t n = 0; n < glyphs.size(); n++ )
     {
       FT_BBox bbox;
       FT_Glyph_Get_CBox(glyphs[n], ft_glyph_bbox_pixels, &bbox);
-      
+
       error = FT_Glyph_To_Bitmap(&glyphs[n],
 				 ft_render_mode_normal,
 				 0,
 				 //&pos[n],
-				 1  //destroy image; 
+				 1  //destroy image;
 				 );
       if (error)
 	throw Py::RuntimeError("Could not convert glyph to bitmap");
-      
+
       FT_BitmapGlyph bitmap = (FT_BitmapGlyph)glyphs[n];
       // now, draw to our target surface (convert position)
-      
+
       //bitmap left and top in pixel, string bbox in subpixel
       FT_Int x = (FT_Int)(bitmap->left-string_bbox.xMin/64.);
       FT_Int y = (FT_Int)(string_bbox.yMax/64.-bitmap->top+1);
       //make sure the index is non-neg
       x = x<0?0:x;
       y = y<0?0:y;
-      
+
       draw_bitmap( &bitmap->bitmap, x, y);
     }
-  
+
   return Py::Object();
 }
 
 
-char FT2Font::get_xys__doc__[] = 
+char FT2Font::get_xys__doc__[] =
 "get_xys()\n"
 "\n"
 "Get the xy locations of the current glyphs\n"
 ;
 Py::Object
 FT2Font::get_xys(const Py::Tuple & args) {
-  
+
   _VERBOSE("FT2Font::get_xys");
   args.verify_length(0);
-  
+
   FT_BBox string_bbox = compute_string_bbox();
   Py::Tuple xys(glyphs.size());
 
@@ -1061,19 +1063,19 @@ FT2Font::get_xys(const Py::Tuple & args) {
 
       FT_BBox bbox;
       FT_Glyph_Get_CBox(glyphs[n], ft_glyph_bbox_pixels, &bbox);
-      
+
       error = FT_Glyph_To_Bitmap(&glyphs[n],
 				 ft_render_mode_normal,
 				 0,
 				 //&pos[n],
-				 1  //destroy image; 
+				 1  //destroy image;
 				 );
       if (error)
 	throw Py::RuntimeError("Could not convert glyph to bitmap");
-      
+
       FT_BitmapGlyph bitmap = (FT_BitmapGlyph)glyphs[n];
 
-      
+
       //bitmap left and top in pixel, string bbox in subpixel
       FT_Int x = (FT_Int)(bitmap->left-string_bbox.xMin/64.);
       FT_Int y = (FT_Int)(string_bbox.yMax/64.-bitmap->top+1);
@@ -1085,11 +1087,11 @@ FT2Font::get_xys(const Py::Tuple & args) {
       xy[1] = Py::Float(y);
       xys[n] = xy;
     }
-  
+
   return xys;
 }
 
-char FT2Font::draw_glyph_to_bitmap__doc__[] = 
+char FT2Font::draw_glyph_to_bitmap__doc__[] =
 "draw_glyph_to_bitmap(x, y, glyph)\n"
 "\n"
 "Draw a single glyph to the bitmap at pixel locations x,y\n"
@@ -1105,30 +1107,30 @@ Py::Object
 FT2Font::draw_glyph_to_bitmap(const Py::Tuple & args) {
   _VERBOSE("FT2Font::draw_glyph_to_bitmap");
   args.verify_length(3);
-  
+
   if (image.width==0 || image.height==0)
     throw Py::RuntimeError("You must first set the size of the bitmap with set_bitmap_size");
-  
+
   long x = Py::Int(args[0]);
   long y = Py::Int(args[1]);
   if (!Glyph::check(args[2].ptr()))
     throw Py::TypeError("Usage: draw_glyph_to_bitmap(x,y,glyph)");
   Glyph* glyph = static_cast<Glyph*>(args[2].ptr());
-  
+
   if ((size_t)glyph->glyphInd >= glyphs.size())
     throw Py::ValueError("glyph num is out of range");
-  
+
   error = FT_Glyph_To_Bitmap(&glyphs[glyph->glyphInd],
 			     ft_render_mode_normal,
 			     0,  //no additional translation
-			     1   //destroy image; 
+			     1   //destroy image;
 			     );
   if (error)
     throw Py::RuntimeError("Could not convert glyph to bitmap");
-  
+
   FT_BitmapGlyph bitmap = (FT_BitmapGlyph)glyphs[glyph->glyphInd];
-  
-  draw_bitmap( &bitmap->bitmap, 
+
+  draw_bitmap( &bitmap->bitmap,
 	       //x + bitmap->left,
 	       x,
 	       //y+bitmap->top
@@ -1146,10 +1148,10 @@ Py::Object
 FT2Font::get_glyph_name(const Py::Tuple & args) {
   _VERBOSE("FT2Font::get_glyph_name");
   args.verify_length(1);
-  
+
   if (!FT_HAS_GLYPH_NAMES(face))
     throw Py::RuntimeError("Face has no glyph names");
-  
+
   char buffer[128];
   if (FT_Get_Glyph_Name(face, (FT_UInt) Py::Int(args[0]), buffer, 128))
     throw Py::RuntimeError("Could not get glyph names.");
@@ -1166,10 +1168,10 @@ Py::Object
 FT2Font::get_charmap(const Py::Tuple & args) {
   _VERBOSE("FT2Font::get_charmap");
   args.verify_length(0);
-  
+
   FT_UInt index;
   Py::Dict charmap;
-  
+
   FT_ULong code = FT_Get_First_Char(face, &index);
   while (index != 0) {
     charmap[Py::Int((int) index)] = Py::Long((long) code);
@@ -1211,7 +1213,7 @@ FT2Font::get_charmap(const Py::Tuple & args) {
 // 4         Full font name       e.g. "New York Bold"
 // 5         Version string       e.g. "August 10, 1991, 1.08d21"
 // 6         Postscript name      e.g. "Times-Bold"
-// 7         Trademark            
+// 7         Trademark
 // 8         Designer             e.g. "Apple Computer"
 
 char FT2Font::get_sfnt__doc__[] =
@@ -1240,26 +1242,27 @@ Py::Object
 FT2Font::get_sfnt(const Py::Tuple & args) {
   _VERBOSE("FT2Font::get_sfnt");
   args.verify_length(0);
-  
-  if (!(face->face_flags & FT_FACE_FLAG_SFNT)) 
+
+  if (!(face->face_flags & FT_FACE_FLAG_SFNT))
     throw Py::RuntimeError("No SFNT name table");
-  
+
   size_t count = FT_Get_Sfnt_Name_Count(face);
-  
+
   Py::Dict names;
   for (size_t j = 0; j < count; j++) {
     FT_SfntName sfnt;
     FT_Error error = FT_Get_Sfnt_Name(face, j, &sfnt);
-    
-    if (error) 
+
+    if (error)
       throw Py::RuntimeError("Could not get SFNT name");
-    
+
     Py::Tuple key(4);
     key[0] = Py::Int(sfnt.platform_id);
     key[1] = Py::Int(sfnt.encoding_id);
     key[2] = Py::Int(sfnt.language_id);
     key[3] = Py::Int(sfnt.name_id);
-    names[key] = Py::String((char *) sfnt.string, (int) sfnt.string_len);
+    names[key] = Py::String((char *) sfnt.string,
+			    (int) sfnt.string_len);
   }
   return names;
 }
@@ -1275,7 +1278,7 @@ FT2Font::get_name_index(const Py::Tuple & args) {
   _VERBOSE("FT2Font::get_name_index");
   args.verify_length(1);
   std::string glyphname = Py::String(args[0]);
-  
+
   return Py::Long((long)
 		  FT_Get_Name_Index(face, (FT_String *) glyphname.c_str()));
 }
@@ -1291,13 +1294,13 @@ FT2Font::get_ps_font_info(const Py::Tuple & args)
   _VERBOSE("FT2Font::get_ps_font_info");
   args.verify_length(0);
   PS_FontInfoRec fontinfo;
-  
+
   FT_Error error = FT_Get_PS_Font_Info(face, &fontinfo);
   if (error) {
     Py::RuntimeError("Could not get PS font info");
     return Py::Object();
   }
-  
+
   Py::Tuple info(9);
   info[0] = Py::String(fontinfo.version);
   info[1] = Py::String(fontinfo.notice);
@@ -1322,20 +1325,20 @@ FT2Font::get_sfnt_table(const Py::Tuple & args) {
   _VERBOSE("FT2Font::get_sfnt_table");
   args.verify_length(1);
   std::string tagname = Py::String(args[0]);
-  
+
   int tag;
   char *tags[] = {"head", "maxp", "OS/2", "hhea",
 		  "vhea", "post", "pclt",  NULL};
-  
+
   for (tag=0; tags[tag] != NULL; tag++)
     if (strcmp(tagname.c_str(), tags[tag]) == 0)
       break;
-  
+
   void *table = FT_Get_Sfnt_Table(face, (FT_Sfnt_Tag) tag);
   if (!table)
     return Py::Object();
   //throw Py::RuntimeError("Could not get SFNT table");
-  
+
   switch (tag) {
   case 0:
     {
@@ -1528,7 +1531,7 @@ Py::Object
 ft2font_module::new_ft2font (const Py::Tuple &args) {
   _VERBOSE("ft2font_module::new_ft2font ");
   args.verify_length(1);
-  
+
   std::string facefile = Py::String(args[0]);
   return Py::asObject( new FT2Font(facefile) );
 }
@@ -1548,7 +1551,7 @@ FT2Font::init_type() {
   _VERBOSE("FT2Font::init_type");
   behaviors().name("FT2Font");
   behaviors().doc("FT2Font");
-  
+
   add_varargs_method("clear", &FT2Font::clear,
 		     FT2Font::clear__doc__);
   add_varargs_method("write_bitmap", &FT2Font::write_bitmap,
@@ -1578,7 +1581,7 @@ FT2Font::init_type() {
 		     FT2Font::set_size__doc__);
   add_varargs_method("set_charmap", &FT2Font::set_charmap,
 		     FT2Font::set_charmap__doc__);
-  
+
   add_varargs_method("get_width_height", &FT2Font::get_width_height,
 		     FT2Font::get_width_height__doc__);
   add_varargs_method("get_descent", &FT2Font::get_descent,
@@ -1597,10 +1600,10 @@ FT2Font::init_type() {
 		     FT2Font::get_ps_font_info__doc__);
   add_varargs_method("get_sfnt_table", &FT2Font::get_sfnt_table,
 		     FT2Font::get_sfnt_table__doc__);
-  add_varargs_method("horiz_image_to_vert_image", 
+  add_varargs_method("horiz_image_to_vert_image",
 		     &FT2Font::horiz_image_to_vert_image,
 		     FT2Font::horiz_image_to_vert_image__doc__);
-  
+
   behaviors().supportGetattr();
   behaviors().supportSetattr();
 };
@@ -1630,7 +1633,7 @@ char ft2font__doc__[] =
 ;
 
 /* Function of no arguments returning new FT2Font object */
-char ft2font_new__doc__[] = 
+char ft2font_new__doc__[] =
 "FT2Font(ttffile)\n"
 "\n"
 "Create a new FT2Font object\n"
@@ -1667,7 +1670,7 @@ void
 initft2font(void)
 {
   static ft2font_module* ft2font = new ft2font_module;
-  
+
   Py::Dict d = ft2font->moduleDictionary();
   d["SCALABLE"] 	= Py::Int(FT_FACE_FLAG_SCALABLE);
   d["FIXED_SIZES"] 	= Py::Int(FT_FACE_FLAG_FIXED_SIZES);
@@ -1685,15 +1688,15 @@ initft2font(void)
   d["KERNING_DEFAULT"]  = Py::Int(FT_KERNING_DEFAULT);
   d["KERNING_UNFITTED"]  = Py::Int(FT_KERNING_UNFITTED);
   d["KERNING_UNSCALED"]  = Py::Int(FT_KERNING_UNSCALED);
-  
-  //initialize library    
-  int error = FT_Init_FreeType( &_ft2Library ); 
-  
-  if (error) 
+
+  //initialize library
+  int error = FT_Init_FreeType( &_ft2Library );
+
+  if (error)
     throw Py::RuntimeError("Could not find initialize the freetype2 library");
 }
 
 ft2font_module::~ft2font_module() {
-  
+
   FT_Done_FreeType( _ft2Library );
 }
